from django.contrib import admin
from django.conf import settings

from rest_framework.authtoken.models import Token
from rest_framework.authtoken.admin import TokenAdmin

from django_tasks import models
from django_tasks.admin_task_actions import AdminTaskActionFactory
from django_tasks.serializers import DocTaskSerializer


class AdminSite(admin.AdminSite):
    site_title = 'Stored Tasks'
    site_header = 'Stored Tasks'
    index_title = 'Index'


site = AdminSite()
site.register(Token, TokenAdmin)


@admin.register(models.DocTask, site=site)
class DocTaskModelAdmin(admin.ModelAdmin):
    change_list_template = 'task_status_display.html'
    list_display = ('name', 'inputs', 'duration', *DocTaskSerializer.Meta.read_only_fields)
    if settings.DEBUG:
        actions = [
           AdminTaskActionFactory.new('doctask_access_test', description='Test async database access'),
           AdminTaskActionFactory.new('doctask_deletion_test', description='Test async deletion'),
        ]

    def has_change_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request):
        return False



import json

import websocket

from django.contrib import admin, messages
from django.core.handlers.asgi import ASGIRequest


class AdminTaskActionFactory:
    header = {'Content-Type': 'application/json'}

    @classmethod
    def new(cls, name, **kwargs):
        @admin.action(**kwargs)
        def action_callable(modeladmin: admin.ModelAdmin, request: ASGIRequest, queryset):
            ws_response = cls.websocket_task_schedule(
                request, name, instance_ids=list(queryset.values_list('pk', flat=True))
            )
            modeladmin.message_user(request, ws_response, messages.INFO)

        action_callable.__name__ = name
        return action_callable

    @classmethod
    def websocket_task_schedule(cls, http_request, task_name, **inputs):
        ws = websocket.WebSocket()
        ws.connect(f'ws://{http_request.get_host()}/tasks/', header=cls.header)
        ws.send(json.dumps([dict(name=task_name, inputs=inputs)], indent=4))
        ws_response = ws.recv()
        ws.close()
        return ws_response



from django.core.asgi import get_asgi_application
from django import urls

from channels.routing import ProtocolTypeRouter, URLRouter
from channels.auth import AuthMiddlewareStack
from channels.security.websocket import AllowedHostsOriginValidator

from rest_framework import routers

from django_tasks.consumers import TaskEventsConsumer, TasksRestConsumer
from django_tasks.viewsets import TaskViewSet


class OptionalSlashRouter(routers.SimpleRouter):
    def __init__(self):
        super().__init__()
        self.trailing_slash = '/?'


asgi_app = get_asgi_application()

drf_router = OptionalSlashRouter()
drf_router.register('tasks', TaskViewSet)

application = ProtocolTypeRouter(
    {
        "http": URLRouter([
            urls.re_path(r'^api/', URLRouter(TasksRestConsumer.get_urls(drf_router))),
            urls.re_path(r'^', asgi_app),
        ]),
        'websocket': AllowedHostsOriginValidator(AuthMiddlewareStack(
            URLRouter([urls.path('tasks/', TaskEventsConsumer.as_asgi())])
        )),
    }
)
